/** \file
 * \brief Example code for Simple Open EtherCAT master
 *
 * Usage : slaveinfo [ifname1]
 * ifname is NIC interface, f.e. eth0
 *
 * This shows the configured slave data.
 *
 * (c)Arthur Ketels 2008 
 */

#include <stdio.h>

#include "ethercattype.h"
#include "nicdrv.h"
#include "ethercatbase.h"
#include "ethercatmain.h"
#include "ethercatconfig.h"
#include "ethercatcoe.h"
#include "ethercatdc.h"

char IOmap[4096];
ec_ODlistt ODlist;
ec_OElistt OElist;

void slaveinfo(char *ifname)
{
	int cnt, i, j;
	uint16 ssigen;
	
	printf("Starting slaveinfo\n");
	
	/* initialise SOEM, bind socket to ifname */
	if (ec_init(ifname))
	{	
		printf("ec_init on %s succeeded.\n",ifname);
		/* find and auto-config slaves */
		if ( ec_config(TRUE, &IOmap) > 0 )
		{
			printf("%d slaves found and configured.\n",ec_slavecount);
			/* wait for all slaves to reach SAFE_OP state */
			ec_statecheck(0, EC_STATE_SAFE_OP,  EC_TIMEOUTSTATE);
			
			ec_configdc();
			
			ec_readstate();
			for( cnt = 1 ; cnt <= ec_slavecount ; cnt++)
			{
				printf("Slave:%d\n Name:%s\n Output size: %dbits\n Input size: %dbits\n State: %d\n Delay: %d[ns]\n Has DC: %d\n",
					   cnt, ec_slave[cnt].name, ec_slave[cnt].Obits, ec_slave[cnt].Ibits,
					   ec_slave[cnt].state, (int)ec_slave[cnt].pdelay, ec_slave[cnt].hasdc);
				printf(" Configured address: %4.4x\n", ec_slave[cnt].configadr);
				printf(" Man: %8.8x ID: %8.8x Rev: %8.8x\n", (int)ec_slave[cnt].eep_man, (int)ec_slave[cnt].eep_id, (int)ec_slave[cnt].eep_rev);
				printf(" SM0 A:%4.4x L:%4d F:%8.8x\n", ec_slave[cnt].SM[0].StartAddr, ec_slave[cnt].SM[0].SMlength, (int)ec_slave[cnt].SM[0].SMflags);
				printf(" SM1 A:%4.4x L:%4d F:%8.8x\n", ec_slave[cnt].SM[1].StartAddr, ec_slave[cnt].SM[1].SMlength, (int)ec_slave[cnt].SM[1].SMflags);
				printf(" SM2 A:%4.4x L:%4d F:%8.8x\n", ec_slave[cnt].SM[2].StartAddr, ec_slave[cnt].SM[2].SMlength, (int)ec_slave[cnt].SM[2].SMflags);
				printf(" SM3 A:%4.4x L:%4d F:%8.8x\n", ec_slave[cnt].SM[3].StartAddr, ec_slave[cnt].SM[3].SMlength, (int)ec_slave[cnt].SM[3].SMflags);
				printf(" SM types 0:%d 1:%d 2:%d 3:%d\n", ec_slave[cnt].SMtype[0], ec_slave[cnt].SMtype[1], ec_slave[cnt].SMtype[2], ec_slave[cnt].SMtype[3]);
				printf(" FMMU0 Ls:%8.8x Ll:%4d Lsb:%d Leb:%d Ps:%4.4x Psb:%d Ty:%2.2x Act:%2.2x\n",
					     (int)ec_slave[cnt].FMMU[0].LogStart, ec_slave[cnt].FMMU[0].LogLength, ec_slave[cnt].FMMU[0].LogStartbit,
					     ec_slave[cnt].FMMU[0].LogEndbit, ec_slave[cnt].FMMU[0].PhysStart, ec_slave[cnt].FMMU[0].PhysStartBit,
					     ec_slave[cnt].FMMU[0].FMMUtype, ec_slave[cnt].FMMU[0].FMMUactive);
				printf(" FMMU1 Ls:%8.8x Ll:%4d Lsb:%d Leb:%d Ps:%4.4x Psb:%d Ty:%2.2x Act:%2.2x\n",
					     (int)ec_slave[cnt].FMMU[1].LogStart, ec_slave[cnt].FMMU[1].LogLength, ec_slave[cnt].FMMU[1].LogStartbit,
					     ec_slave[cnt].FMMU[1].LogEndbit, ec_slave[cnt].FMMU[1].PhysStart, ec_slave[cnt].FMMU[1].PhysStartBit,
					     ec_slave[cnt].FMMU[1].FMMUtype, ec_slave[cnt].FMMU[1].FMMUactive);
				printf(" FMMU2 Ls:%8.8x Ll:%4d Lsb:%d Leb:%d Ps:%4.4x Psb:%d Ty:%2.2x Act:%2.2x\n",
					     (int)ec_slave[cnt].FMMU[2].LogStart, ec_slave[cnt].FMMU[2].LogLength, ec_slave[cnt].FMMU[2].LogStartbit,
					     ec_slave[cnt].FMMU[2].LogEndbit, ec_slave[cnt].FMMU[2].PhysStart, ec_slave[cnt].FMMU[2].PhysStartBit,
					     ec_slave[cnt].FMMU[2].FMMUtype, ec_slave[cnt].FMMU[2].FMMUactive);
				printf(" FMMUfunc 0:%d 1:%d 2:%d 3:%d\n",
				         ec_slave[cnt].FMMU0func, ec_slave[cnt].FMMU2func, ec_slave[cnt].FMMU2func, ec_slave[cnt].FMMU3func);
				printf(" MBX length: %d MBX protocols : %2.2x\n", ec_slave[cnt].mbx_l, ec_slave[cnt].mbx_proto);
				ssigen = ec_siifind(cnt, ECT_SII_GENERAL);
				/* SII general section */
				if (ssigen)
                {
					ec_slave[cnt].CoEdetails = ec_siigetbyte(cnt, ssigen + 0x07);
					ec_slave[cnt].FoEdetails = ec_siigetbyte(cnt, ssigen + 0x08);
					ec_slave[cnt].EoEdetails = ec_siigetbyte(cnt, ssigen + 0x09);
					ec_slave[cnt].SoEdetails = ec_siigetbyte(cnt, ssigen + 0x0a);
					if((ec_siigetbyte(cnt, ssigen + 0x0d) & 0x02) > 0)
					{
						ec_slave[cnt].blockLRW = 1;
						ec_slave[0].blockLRW++;						
					}	
					ec_slave[cnt].Ebuscurrent = ec_siigetbyte(cnt, ssigen + 0x0e);
					ec_slave[cnt].Ebuscurrent += ec_siigetbyte(cnt, ssigen + 0x0f) << 8;
					ec_slave[0].Ebuscurrent += ec_slave[cnt].Ebuscurrent;
                }
				printf(" CoE details: %2.2x FoE details: %2.2x EoE details: %2.2x SoE details: %2.2x\n",
					     ec_slave[cnt].CoEdetails, ec_slave[cnt].FoEdetails, ec_slave[cnt].EoEdetails, ec_slave[cnt].SoEdetails);
				printf(" Ebus current: %d[mA]\n only LRD/LWR:%d\n",
					     ec_slave[cnt].Ebuscurrent, ec_slave[cnt].blockLRW);
				if ((ec_slave[cnt].mbx_proto & 0x04))
				{
					ODlist.Entries = 0;
					if( ec_readODlist(cnt, &ODlist));
					{
						printf(" CoE Object Description found, %d entries.\n",ODlist.Entries);
						for( i = 0 ; i < ODlist.Entries ; i++)
						{
							ec_readODdescription(i, &ODlist); 
							printf(" Index: %4.4x Datatype: %4.4x Objectcode: %2.2x Name: %s\n",
								   ODlist.Index[i], ODlist.DataType[i], ODlist.ObjectCode[i], ODlist.Name[i]);
							ec_readOE(i, &ODlist, &OElist);
							for( j = 0 ; j <= ODlist.MaxSub[i] ; j++)
							{
								printf("  Sub: %2.2x Datatype: %4.4x Bitlength: %4.4x Obj.access: %4.4x Name: %s\n",
									   j, OElist.DataType[j], OElist.BitLength[j], OElist.ObjAccess[j], OElist.Name[j]);
							}	
						}	
					}	
				}	
			}	
		}
		else
		{
			printf("No slaves found!\n");
		}
		printf("End slaveinfo, close socket\n");
		/* stop SOEM, close socket */
		ec_close();
	}
	else
	{
		printf("No socket connection on %s\nExcecute as root\n",ifname);
	}	
}	

int main(int argc, char *argv[])
{
	printf("SOEM (Simple Open EtherCAT Master)\nSlaveinfo\n");
	
	if (argc > 1)
	{		
		/* start slaveinfo */
		slaveinfo(argv[1]);
	}
	else
	{
		printf("Usage: slaveinfo ifname\nifname = eth0 for example\n");
	}	
	
	printf("End program\n");
	return (0);
}
